import CloudsOfArx.scrape as scrape
import CloudsOfArx.create_image as create_image

def create_wordcloud(ADS_TOKEN, author, image_file=None, orcid=None, save_name=None, test_case=False):
    """
    A function to create a wordcloud image out of the abstracts of an author's first-authored papers.

    ADS_TOKEN: [string] - a token necessary to use the NASA ADS API.
    author: [string] - first-author whose work is desired string format 'LastName, FirstName'
    image_file: [string] - the string representing the file location of the image desired.
    orcid: [string] - optional parameter to remove degeneracies between authors with the same name.
    save_name: [string] - optional parameter to change the save name of the wordcloud image.

    returns: None
    """
    #format the abstracts of the first-authored papers
    first_author_papers = scrape.get_papers(ADS_TOKEN, author, orcid)
    abstract_string = scrape.compile_abstracts(first_author_papers)

    #generate the wordcloud image
    create_image.create_wordcloud(abstract_string, image_file, test_case, save_name)

    print("Your wordcloud creation was succesful!")
    return True


def test_print():
    print("This package is working!")
    return True




import ads
from operator import itemgetter
from bs4 import BeautifulSoup
import requests
import re

def clean_results(papers_dict):
    """
    A function to remove any unwanted publications from the scraped papers from NASA ADS.

    papers_dict: [dict] - dictionary of scraped meta data for research papers by the desired author.

    returns [dict] - cleaned version of papers_dict
    """
    first_author_papers = []
    for paper in papers_dict:
        if paper['pub'] != 'GRB Coordinates Network':
            first_author_papers.append(paper)

    return first_author_papers

def get_papers(ADS_TOKEN, author, orcid=None):
    """
    A function to scrape NASA ADS for first-author papers of a given author.

    ADS_TOKEN: [string] - a token necessary to use the NASA ADS API.
    author: [string] - first-author whose work is desired string format 'LastName, FirstName'
    orcid: [string] - optional parameter to remove degeneracies between authors with the same name.

    returns [dict] - a dictionary populated with the papers by the author.
    """

    # CONFIGURE ADS
    ads.config.token = ADS_TOKEN
        
    if orcid is not None:
        papers = list(
            ads.SearchQuery(
                first_author=author,
                orcid=orcid,
                fl=[
                    "id",
                    "title",
                    "author",
                    "doi",
                    "year",
                    "pubdate",
                    "pub",
                    "volume",
                    "page",
                    "identifier",
                    "doctype",
                    "citation_count",
                    "bibcode"], max_pages=100))
    else:
        papers = list(
            ads.SearchQuery(
                first_author=author,
                fl=[
                    "id",
                    "title",
                    "author",
                    "doi",
                    "year",
                    "pubdate",
                    "pub",
                    "volume",
                    "page",
                    "identifier",
                    "doctype",
                    "citation_count",
                    "bibcode"], max_pages=100))
        
    dicts = []
    for paper in papers:
        aid = [
            ":".join(t.split(":")[1:])
            for t in paper.identifier
            if t.startswith("arXiv:")
        ]
        for t in paper.identifier:
            if len(t.split(".")) != 2:
                continue
            try:
                list(map(int, t.split(".")))
            except ValueError:
                pass
            else:
                aid.append(t)
        try:
            page = int(paper.page[0])
        except (ValueError, TypeError):
            page = None
            if paper.page is not None and paper.page[0].startswith("arXiv:"):
                aid.append(":".join(paper.page[0].split(":")[1:]))
        dicts.append(
            dict(
                doctype=paper.doctype,
                year=paper.year,
                pubdate=paper.pubdate,
                doi=paper.doi[0] if paper.doi is not None else None,
                pub=paper.pub,
                volume=paper.volume,
                page=page,
                arxiv=aid[0] if len(aid) else None,
                citations=(
                    paper.citation_count
                    if paper.citation_count is not None
                    else 0
                ),
                url="https://ui.adsabs.harvard.edu/abs/" + paper.bibcode,
            )
        )
    papers_dict = sorted(dicts, key=itemgetter("pubdate"), reverse=True)
    cleaned_papers_dict = clean_results(papers_dict)

    return cleaned_papers_dict


def compile_abstracts(first_author_papers):
    """
    A function to compile all the abstracts from the desired first-author papers to be 
    used in the word cloud. We use beautifulsoup to parse through the NASA ADS abstract page
    to find the desired abstracts. 

    first_author_papers: [dict] - dictionary of scraped meta data for research papers by the desired author.

    returns [string] - a string of all the text within the desired abstracts.
    """

    abstracts = []
    for i in range(len(first_author_papers)):
        #retrieve the ADS abstract page
        URL = first_author_papers[i]['url']
        
        # scrape the ADS abstract page
        page = requests.get(URL)
        
        #parse the page and pull out only the abstract text (excluding inline latex)
        soup = BeautifulSoup(page.content, 'html.parser')
        abstract = soup.p.string
        pattern = r'\$(.*?)\$'
        formatted_abstract = re.sub(pattern, '', abstract[21:-20])
        abstracts.append(formatted_abstract)

    #turn the list of abstracts into one long string
    total_abstract_string = ''
    for abstract in abstracts:
        total_abstract_string += abstract + ' '
        
    return total_abstract_string

    


import numpy as np
from PIL import Image
from os import path
import matplotlib.pyplot as plt
from scipy.ndimage import gaussian_gradient_magnitude
from wordcloud import WordCloud, ImageColorGenerator


def create_image_mask(image_file=None):
    """
    A function which takes in an image and formats a mask to be used by the wordcloud.

    image_file: [string] - the string representing the file location of the image desired.

    returns: mask, original_image

    """
    if image_file is not None:
        # choose image we wish to use
        image = np.array(Image.open(path.join(image_file)))
        image = image[:,:,:3]
        
        # create mask white is "masked out" using edge detection
        mask = image
        mask[mask.sum(axis=2) == 0] = 255
        edges = np.mean([gaussian_gradient_magnitude(image[:, :, i] / 255., 2) for i in range(3)], axis=0)
        mask[edges > .08] = 255

        return mask, image
    else:
        # Just use a bland template
        image = np.ones((1200, 1600, 3), dtype=int) * 255
        mask = np.zeros((1200, 1600, 3), dtype=int) 

        return mask, image


def create_wordcloud(text_data, image_file, test_case, save_name=None,):
    """
    A function which generates a wordcloud from a mask and text data.

    text_data: [string] - a string which contains all the relevant data needed to make the 
    word cloud.
    image_file: [string] - the string representing the file location of the image desired.
    save_name: [string] - optional parameter to change the save name of the wordcloud image.

    returns: None
    """

    # generate the mask
    mask_data, original_image = create_image_mask(image_file)

    # generate word cloud
    wc = WordCloud(max_words=2000, mask=mask_data, max_font_size=40, random_state=42, relative_scaling=0)
    wc.generate(text_data)

    # create coloring from image
    image_colors = ImageColorGenerator(original_image)
    wc.recolor(color_func=image_colors)
    
    plt.imshow(wc, interpolation="bilinear")
    plt.xticks([])
    plt.yticks([])

    if test_case is False:
        if save_name is not None:
            wc.to_file(save_name + '.png')
        else:
            wc.to_file("wordcloud.png")
    else: 
        return True






"""A setuptools based setup module.
See:
https://packaging.python.org/guides/distributing-packages-using-setuptools/
https://github.com/pypa/sampleproject
"""

# Always prefer setuptools over distutils
from setuptools import setup
import pathlib

here = pathlib.Path(__file__).parent.resolve()

setup(
    name='CloudsOfArx',  # Required
    version='0.3.0',  # Required
    description='Web Scraper package for generating wordclouds from research paper abstracts.',  # Optional
    long_description=(here / 'README.md').read_text(encoding='utf-8'),  # Optional
    long_description_content_type='text/markdown',  # Optional (see note above)
    url='https://github.com/James11222/CloudsOfArx/',  # Optional

    author='James Sunseri',  # Optional

    author_email='jamessunseri@berkeley.edu',  # Optional

    classifiers=[  # Optional
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 4 - Beta',

        # Indicate who your project is intended for
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        # Pick your license as you wish
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        "Programming Language :: Python :: 3.10",
        'Programming Language :: Python :: 3 :: Only'],

    keywords=['ArXiv', 'ADS', 'wordcloud'],  # Optional
    packages=['CloudsOfArx'],
    platforms=['any'],
    license="MIT",
    python_requires='>=3.6, <4',
    setup_requires=['pytest-runner'],
    # install_requires=['requests'],  # Optional
    tests_require=['pytest'],
    project_urls={  # Optional
        'Bug Reports': 'https://github.com/James11222/CloudsOfArx/issues',
        'Funding': 'https://donate.pypi.org',
        'Check out my Website!': 'http://www.jamessunseri.com',
        'Source': 'https://github.com/James11222/CloudsOfArx/',
    }
)